#!/bin/bash -vx

#
# Copyright (C) 2021 University of Southern California.
# All rights reserved.                                            
#                                                                
# Redistribution and use in source and binary forms are permitted
# provided that the above copyright notice and this paragraph are
# duplicated in all such forms and that any documentation, advertising
# materials, and other materials related to such distribution and use
# acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from
# this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# 

set -e

outage_dataset=$1
maxmind_blocks=$2
supplementalds=$3

#tail subtracts 1
maxsplit=$(ls $outage_dataset/input/{reprobing*txt.bz2,ips*.t*xt*bz2} 2>/dev/null | tail -n+2 |  wc -l)
[ "$maxsplit" == 4 ] || [ "$maxsplit" == 3 ] || {
    echo "ERROR: expected to have 4 (indexed 0..3 for older datasets) or "
    echo "       5 (indexed 0..4 for newer datasets) splits, but found only $((maxsplit+1))"
    exit 1
} >&2

for i in $(seq 0 $maxsplit); do
    {
        #allow different naming
        f=$(ls $outage_dataset/input/{reprobing*.$i.*txt.bz2,ips*.t*xt*.$i.*bz2} 2>/dev/null) || true
        cnt=$(echo $f | wc -w)
        [ "$cnt" == "1" ] || {
                echo "ERROR: didn't find blocks $i in $outage_dataset/input/" >&1
                exit 1
        }
        echo "#fsdb -F t prefix"
        bzcat $f
    } | dbroweval '_prefix=~s%/.*%%; _prefix = sprintf("%02x"x4, split("\\.", _prefix))' |
        dbcolrename prefix block | dbsort block | dbcolcreate -e '+' present >/tmp/blocks.fsdb
    #effectively compute left join
    dbjoin --type outer -S \
           --input /tmp/blocks.fsdb \
           --input $maxmind_blocks block |
		dbrow '_present eq "+"' |
		dbcol block IP latitude longitude accuracy_radius country region city metro_code color \
           > $supplementalds/blocks.$i.pass-1.fsdb

    rm -f /tmp/blocks.fsdb
done
    
eval ls -1 $supplementalds/blocks.{0..$maxsplit}.pass-1.fsdb |
    dbmerge --xargs block |
    sed '2,$ { /^#/ d; }' |
    bzip2 \
        > $supplementalds/block_to_geo.fsdb.bz2
    
bzcat $supplementalds/block_to_geo.fsdb.bz2 |
    dbrow '_country eq "US"' |
    bzip2 \
        > $supplementalds/block_to_geo.US.fsdb.bz2
