#!/bin/bash

#
# Copyright (C) 2021 University of Southern California.
# All rights reserved.                                            
#                                                                
# Redistribution and use in source and binary forms are permitted
# provided that the above copyright notice and this paragraph are
# duplicated in all such forms and that any documentation, advertising
# materials, and other materials related to such distribution and use
# acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from
# this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# 

set -e
#This is how to Build a supplemental dataset
MM=$1
DS=$2
SD=$3
#These need to be set and directories must exist

if [ -z "$MM" ] || [ -z "$DS" ] || [ -z "$SD" ]; then
   echo "Usage: $0 <maxmind_dir> <dataset_dir> <output_dir>"
   echo 
   echo "Where:"
   echo "  <maxminddir> is the directory that contains compressed CSV files downloaded from Maxmind"
   echo "    GeoLite2-City-Locations*.csv.bz2 (or .gz) and"
   echo "    GeoLite2-City-Blocks-IPv4-*.csv.bz2 (or .gz)"
   echo ""
   echo "  <dataset_dir> is the directory containing USC/ISI internet_outage_adaptive-dataset"
   echo ""
   echo "  <output_dir> is the output directory where geolocation for the dataset's blocks is written"
   echo ""
   echo "Example:"
   echo "  $0 maxmind_dir /traces/outages/internet_outage_adaptive_a23w-20160101 \\"
   echo "      /traces/outages/internet_outage_adaptive_a23_supplement-20160101"
   echo ""
   exit 1
fi
MM=$(readlink -f $MM)
DS=$(readlink -f $DS)
SD=$(readlink -f $SD)

CODE=$(dirname $(readlink -f $0))
cd $CODE

echo 1. Parse/join maxmind CSV data:

skip=false
#was v1 before 2019
#./parse_geolitecity-v1.pl $MM > $SD/maxmind_parsed.fsdb

if [ -e $SD/maxmind_parsed.fsdb ]; then
    read -p "Overwrite $SD/maxmind_parsed.fsdb (Y/n) " yn
    if [ "$yn" == "n" ] || [ "$yn" == "N" ]; then
        echo "...reusing existing"
        skip=true
    fi
fi
$skip || { ./parse_geolitecity-v2.pl $MM > $SD/maxmind_parsed.fsdb; }

echo 2. Convert ip ranges to blocks:
skip=false
if [ -e $SD/maxmind_blocks.fsdb ]; then
    read -p "Overwrite $SD/maxmind_blocks.fsdb (Y/n) " yn
    if [ "$yn" == "n" ] || [ "$yn" == "N" ]; then
        echo "...reusing existing"
        skip=true
    fi
fi
$skip || ./blockify.sh < $SD/maxmind_parsed.fsdb > $SD/maxmind_blocks.fsdb

echo 3. Join inputs and produce outputs:

./join_inputs.sh $DS $SD/maxmind_blocks.fsdb $SD

echo 4. Cleanup:

#rm -f $SD/*.fsdb
