#!/usr/bin/perl -w

#
# Copyright (C) 2021 University of Southern California.
# All rights reserved.                                            
#                                                                
# Redistribution and use in source and binary forms are permitted
# provided that the above copyright notice and this paragraph are
# duplicated in all such forms and that any documentation, advertising
# materials, and other materials related to such distribution and use
# acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from
# this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# 

use strict;
use warnings;
use Text::CSV_PP;
use Encode qw(encode);

my $mmdir = $ARGV[0];

my $COMPRESSION="gz";
#my $COMPRESSION="bz2";

my $catter;
my $extension;
if ($COMPRESSION eq "gz") {
    $catter = "zcat";
    $extension = "gz";
} elsif ($COMPRESSION eq "bz2") {
    $catter = "bzcat";
    $extension="bz2";
}

#my @ba = glob("$mmdir/GeoLiteCity-Blocks*.csv.$extension");
my @ba = glob("$mmdir/GeoLite2-City-Blocks-IPv4-*.csv.$extension");
die "GeoLiteCity-Blocks: @ba"
	if $#ba != 0;
my $blocks = $ba[0]; 
#my @la = glob("$mmdir/GeoLiteCity-Location*.csv.$extension");
my @la = glob("$mmdir/GeoLite2-City-Locations*.csv.$extension");
die "GeoLiteCity-Location: @la"
	if $#la != 0;
my $loc_id = $la[0];

my $FSDB_HDR = "#fsdb -F t ip_low ip_high latitude longitude accuracy_radius country region city metro_code";
my $LOC_HEADERS="geoname_id,locale_code,continent_code,continent_name,country_iso_code,country_name,subdivision_1_iso_code,subdivision_1_name,subdivision_2_iso_code,subdivision_2_name,city_name,metro_code,time_zone,is_in_european_union";

my $BLK_HEADERS="network,geoname_id,registered_country_geoname_id,represented_country_geoname_id,is_anonymous_proxy,is_satellite_provider,postal_code,latitude,longitude,accuracy_radius";
my @LOC_COLUMNS;
my @BLK_COLUMNS;


my $parser = Text::CSV_PP->new({binary => 1});

open(my $fh_loc, "$catter $loc_id |") || die ("Can't read from $loc_id");
my @LOCATIONS;
my $UNKNOWN_LOCATION;

while (my $line = <$fh_loc>) {
    if ($line =~ /^\d/) {
        $parser->parse($line) || die "failed to parse line '$line'\n";
        my @fields = $parser->fields(); 
        #print "@fields", "\n";
        for(my $i=0; $i<@fields; ++$i) {
            $fields[$i] = "-"
                if !defined($fields[$i]) || $fields[$i] eq "";
        }
        #CSV:
        #geoname_id(0)                  5370542
        #locale_code(1)                 en
        #continent_code(2)              NA
        #continent_name(3)              "North America"
        #country_iso_code(4)            US
        #country_name(5)                "United States"
        #subdivision_1_iso_code(6)      CA
        #subdivision_1_name(7)          California
        #subdivision_2_iso_code(8)      ""
        #subdivision_2_name(9)          ""
        #city_name(10)                  "Marina del Rey"
        #metro_code(11)                 803
        #time_zone(12)                  America/Los_Angeles
        #is_in_european_union(13)       0
        #FSDB:
        #fsdb -F t ip_low ip_high latitude longitude accuracy_radius =>  country region city metro_code
        $LOCATIONS[$fields[0]] = encode('UTF-8', join("\t", @fields[4,6,10,11]),
                                        Encode::FB_CROAK);
    } elsif ($line =~ /^Copyright/) {
        next;
    } elsif ($line eq "$LOC_HEADERS\n") {
        @LOC_COLUMNS = split(",", $LOC_HEADERS);
        next;
    } else {
        die("Can't parse $loc_id: $line");
    }
    push @LOCATIONS, join("\t", ("-")x4);
    $UNKNOWN_LOCATION=$#LOCATIONS;
}

print "$FSDB_HDR\n";

open(my $fh_blk, "$catter $blocks |") || die ("Can't read from $blocks");
while (my $line = <$fh_blk>) {
    if ($line =~ /^\d+\.\d+\./) {
        $parser->parse($line);
        my @fields = $parser->fields();
        #network(0)
        #geoname_id(1)
        #registered_country_geoname_id(2)
        #represented_country_geoname_id(3)
        #is_anonymous_proxy(4)
        #is_satellite_provider(5)
        #postal_code(6)
        #latitude(7)
        #longitude(8)
        #accuracy_radius(9)
        my ($blk_add, $blk_len) = split("/", $fields[0]);
        my @os = split(/\./, $blk_add);
        die "Error parsing IP $blk_add\n"
                if @os != 4;
        my $ip_l = ($os[0] << 24) | ($os[1] << 16) | ($os[2] << 8) | $os[3];
        my $ip_m = 2**(32-$blk_len) -1;
        my $ip_h = $ip_l | $ip_m;

        die "Leaking bits in $blk_add/$blk_len\n"
            if ($ip_l & $ip_m) != 0;
        my $loc =$fields[1];
       
        my $lat = $fields[7];
        my $lon = $fields[8];
        if ($lat eq "" or $lon eq "") {
            print STDERR "** skipping block $blk_add/$blk_len with unknown locations ll:($lat, $lon)\n";
            next;
        } 
        my $rad = $fields[9]; $rad="-" if $rad eq "";
        $loc = $UNKNOWN_LOCATION 
            if ($loc eq "");
        my $rst = join("\t", $ip_l, $ip_h, $lat, $lon, $rad, $LOCATIONS[$loc]);
        print "$rst\n";
    } elsif ($line =~ /^Copyright/) {
        next;
    } elsif ($line eq "$BLK_HEADERS\n") {
        @BLK_COLUMNS = split(",", $BLK_HEADERS);
        next;
    } else {
        die("Can't parse $blocks: $line");
    }
}
